@@ -33,7 +33,7 @@
 
 
 /*
- * $Id: dproto.h,v 1.10 2005/05/11 12:52:50 abe Exp $
+ * $Id: dproto.h,v 1.12 2008/04/15 13:31:47 abe Exp $
  */
 
 
@@ -46,6 +46,7 @@ _PROTOTYPE(extern char *get_nlist_path,(int ap));
 #endif	/* !defined(N_UNIX) */
 
 _PROTOTYPE(extern int is_file_named,(char *p, int cd));
+_PROTOTYPE(extern void process_socket,(KA_T sa));
 _PROTOTYPE(extern struct l_vfs *readvfs,(KA_T vm));
 
 #if	defined(HASKQUEUE)
@@ -57,5 +58,5 @@ _PROTOTYPE(extern void process_pipe,(KA_T pa));
 #endif	/* FREEBSDV>=2020 */
 
 #if	defined(HAS9660FS)
-_PROTOTYPE(extern int read_iso_node,(struct vnode *v, dev_t *d, int *dd, unsigned long *ino, long *nl, unsigned long *sz));
+_PROTOTYPE(extern int read_iso_node,(struct vnode *v, dev_t *d, int *dd, INODETYPE *ino, long *nl, SZOFFTYPE *sz));
 #endif	/* defined(HAS9660FS) */
