@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.16 2005/05/11 12:53:27 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.21 2008/04/15 13:32:26 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -42,19 +42,105 @@ static char *rcsid = "$Id: dproc.c,v 1.16 2005/05/11 12:53:27 abe Exp $";
  * Local definitions
  */
 
+#define	FDINFO_FLAGS		1	/* fdinfo flags available */
+#define	FDINFO_POS		2	/* fdinfo position available */
+#define FDINFO_ALL		(FDINFO_FLAGS | FDINFO_POS)
 #define	LSTAT_TEST_FILE		"/"
 #define LSTAT_TEST_SEEK		1
 
+#if	!defined(ULLONG_MAX)
+#define	ULLONG_MAX		18446744073709551615ULL
+#endif	/* !defined(ULLONG_MAX) */
+
+
+/*
+ * Local structures
+ */
+
+struct l_fdinfo {
+	int flags;			/* flags: line value */
+	off_t pos;			/* pos: line value */
+};
+
+
+/*
+ * Local variables
+ */
+
+short Cscko, Scko;			/* socket processing options */
+struct lproc *PLp;			/* process' Lp */
+
 
 /*
  * Local function prototypes
  */
 
-_PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl, int *slash));
+_PROTOTYPE(static int get_fdinfo,(char *p, struct l_fdinfo *fi));
+_PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl));
+_PROTOTYPE(static int nm2id,(char *nm, int *id, int *idl));
+_PROTOTYPE(static int read_id_stat,(int ty, char *p, int id, char **cmd,
+				    int *ppid, int *pgid));
 _PROTOTYPE(static void process_proc_map,(char *p, struct stat *s, int ss));
-_PROTOTYPE(static int read_proc_stat,(char *p, int pid, char **cmd, int *ppid,
-	   int *pgid));
+_PROTOTYPE(static int process_id,(char *idp, int idpl, char *cmd, UID_ARG uid,
+				  int pid, int ppid, int pgid));
 _PROTOTYPE(static int statEx,(char *p, struct stat *s, int *ss));
+ 
+
+#if	defined(HASSELINUX)
+_PROTOTYPE(static int cmp_cntx_eq,(char *pcntx, char *ucntx));
+
+
+#include <fnmatch.h>
+
+
+/*
+ * cmp_cntx_eq -- compare program and user security contexts
+ */
+
+static int
+cmp_cntx_eq(pcntx, ucntx)
+	char *pcntx;			       /* program context */
+	char *ucntx;			       /* user supplied context */
+{
+	return !fnmatch(ucntx, pcntx, 0);
+}
+
+
+/*
+ * enter_cntx_arg() - enter name ecurity context argument
+ */
+
+int
+enter_cntx_arg(cntx)
+	char *cntx;			       /* context */
+{
+	cntxlist_t *cntxp;
+/*
+ * Search the argument list for a duplicate.
+ */
+	for (cntxp = CntxArg; cntxp; cntxp = cntxp->next) {
+	    if (!strcmp(cntxp->cntx, cntx)) {
+		if (!Fwarn) {
+		    (void) fprintf(stderr, "%s: duplicate context: %s\n",
+			Pn, cntx);
+		}
+		return(1);
+	    }
+	}
+/*
+ * Create and link a new context argument list entry.
+ */
+	if (!(cntxp = (cntxlist_t *)malloc((MALLOC_S)sizeof(cntxlist_t)))) {
+	    (void) fprintf(stderr, "%s: no space for context: %s\n", Pn, cntx);
+	    Exit(1);
+	}
+	cntxp->f = 0;
+	cntxp->cntx = cntx;
+	cntxp->next = CntxArg;
+	CntxArg = cntxp;
+	return(0);
+}
+#endif	/* defined(HASSELINUX) */
 
 
 /*
@@ -64,31 +150,29 @@ _PROTOTYPE(static int statEx,(char *p, struct stat *s, int *ss));
 void
 gather_proc_info()
 {
-	static char *cmd = (char *)NULL;
-	char *cp;
-	short cscko, lnk, pn, pss, sf, scko;
-	struct dirent *dp, *fp;
-	static char *dpath = (char *)NULL;
-	static int dpathl = 0;
-	int enls, enss, f, fd, i, ls, n, slash, ss, sv;
-	DIR *fdp;
-	struct stat lsb, sb;
-	char nmabuf[MAXPATHLEN + 1], pbuf[MAXPATHLEN + 1];
+	char *cmd;
+	struct dirent *dp;
+	struct stat sb;
+	int lwp, n, nl, pgid, pid, ppid, rv, tx;
+	static char *lwppath = (char *)NULL;
+	static int lwppathl = 0;
 	static char *path = (char *)NULL;
 	static int pathl = 0;
-	int pgid, pid, ppid;
 	static char *pidpath = (char *)NULL;
 	static MALLOC_S pidpathl = 0;
 	static MALLOC_S pidx = 0;
 	static DIR *ps = (DIR *)NULL;
-	int txts = 0;
+	static char *taskpath = (char *)NULL;
+	static int taskpathl = 0;
+	DIR *ts;
 	UID_ARG uid;
+
 /*
  * Do one-time setup.
  */
 	if (!pidpath) {
 	    pidx = strlen(PROCFS) + 1;
-	    pidpathl = pidx + 64 + 1;		/* 64 is growth room */
+	    pidpathl = pidx + 64 + 1;	/* 64 is growth room */
 	    if (!(pidpath = (char *)malloc(pidpathl))) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for \"%s/\"<pid>\n",
@@ -123,8 +207,8 @@ gather_proc_info()
 	     * Selection ANDing or only network file options have been
 	     * specified, so set unconditional skipping of regular files.
 	     */
-		cscko = 0;
-		scko = 1;
+		Cscko = 0;
+		Scko = 1;
 	    } else {
 
 	    /*
@@ -136,9 +220,9 @@ gather_proc_info()
 	     * enable conditional file skipping.
 	     */
 		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
-		    cscko = scko = 0;
+		    Cscko = Scko = 0;
 		else
-		    cscko = scko = 1;
+		    Cscko = Scko = 1;
 	    }
 	} else {
 
@@ -146,7 +230,7 @@ gather_proc_info()
 	 * No network file selection options were specified.  Enable
 	 * unconditional file processing.
 	 */
-	    cscko = scko = 0;
+	    Cscko = Scko = 0;
 	}
 /*
  * Read /proc, looking for PID directories.  Open each one and
@@ -160,28 +244,13 @@ gather_proc_info()
 	} else
 	    (void) rewinddir(ps);
 	while ((dp = readdir(ps))) {
-	    for (f = n = pid = 0, cp = dp->d_name; *cp; cp++) {
-
-#if	defined(__STDC__)	/* { */
-		if (!isdigit((unsigned char)*cp))
-#else	/* !defined(__STDC__)	   } { */
-		if (!isascii(*cp) || !isdigit((unsigned char)*cp))
-#endif	/* defined(__STDC__)	   } */
-
-		{
-		    f = 1;
-		    break;
-		}
-		pid = pid * 10 + (*cp - '0');
-		n++;
-	    }
-	    if (f)
+	    if (nm2id(dp->d_name, &pid, &n))
 		continue;
 	/*
 	 * Build path to PID's directory.
 	 */
 	    if ((pidx + n + 1 + 1) > pidpathl) {
-		pidpathl = pidx + n + 64 + 1;
+		pidpathl = pidx + n + 1 + 1 + 64;
 		if (!(pidpath = (char *)realloc((MALLOC_P *)pidpath, pidpathl)))
 		{
 		    (void) fprintf(stderr,
@@ -198,303 +267,162 @@ gather_proc_info()
 	    if (stat(pidpath, &sb))
 		continue;
 	    uid = (UID_ARG)sb.st_uid;
+	/*
+	 * Process the PID's process information.
+	 */
 	    (void) make_proc_path(pidpath, n, &path, &pathl, "stat");
-	    if (read_proc_stat(path, pid, &cmd, &ppid, &pgid)
-	    ||  is_proc_excl(pid, pgid, uid, &pss, &sf)
-	    ||  is_cmd_excl(cmd, &pss, &sf))
+	    rv = read_id_stat(0, path, pid, &cmd, &ppid, &pgid);
+	    if (rv == 1)
 		continue;
-	    if (cscko) {
-		if (sf & SELPROC)
-		    scko = 0;
-		else
-		    scko = 1;
-	    }
-	    alloc_lproc(pid, pgid, ppid, uid, cmd, (int)pss, (int)sf);
-	    Plf = (struct lfile *)NULL;
-	/*
-	 * Process the PID's current working directory info.
-	 */
-	    (void) make_proc_path(pidpath, n, &path, &pathl, "cwd");
-	    alloc_lfile(CWD, -1);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-		if (!Fwarn) {
-		    (void) memset((void *)&sb, 0, sizeof(sb));
-		    lnk = ss = 0;
-		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-			strerror(errno));
-		    nmabuf[sizeof(nmabuf) - 1] = '\0';
-		    (void) add_nma(nmabuf, strlen(nmabuf));
-		    pn = 1;
-		} else
-		    pn = 0;
-	    } else {
-		lnk = 1;
-		if ((scko && !slash) || !scko) {
-		    pn = 1;
-		    ss = SB_ALL;
-		    if (HasNFS) {
-			if (sv = statsafely(path, &sb)) {
-			    if (lnk)
-				sv = statEx(pbuf, &sb, &ss);
-			}
-		    } else
-			sv = stat(path, &sb);
-		    if (sv) {
-			ss = 0;
-			if (!Fwarn) {
-			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-				strerror(errno));
-			    nmabuf[sizeof(nmabuf) - 1] = '\0';
-			    (void) add_nma(nmabuf, strlen(nmabuf));
-			}
-		    }
-		} else
-		    pn = 0;
-	    }
-	    if (pn) {
-		(void) process_proc_node(lnk ? pbuf : path,
-					 &sb, ss,
-					 (struct stat *)NULL, 0);
-		if (Lf->sf)
-		    link_lfile();
-	    }
-	/*
-	 * Process the PID's root directory info.
-	 */
-	    (void) make_proc_path(pidpath, n, &path, &pathl, "root");
-	    alloc_lfile(RTD, -1);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-		if (!Fwarn) {
-		    (void) memset((void *)&sb, 0, sizeof(sb));
-		    lnk = ss = 0;
-		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-			strerror(errno));
-		    nmabuf[sizeof(nmabuf) - 1] = '\0';
-		    (void) add_nma(nmabuf, strlen(nmabuf));
-		    pn = 1;
-		} else
-		    pn = 0;
-	    } else {
-		lnk = 1;
-		if ((scko && !slash) || !scko) {
-		    pn = 1;
-		    if (HasNFS) {
-			if (sv = statsafely(path, &sb)) {
-			    if (lnk)
-				sv = statEx(pbuf, &sb, &ss);
-			}
-		    } else
-			sv = stat(path, &sb);
-		    if (sv) {
-			ss = 0;
-			if (!Fwarn) {
-			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-				strerror(errno));
-			    nmabuf[sizeof(nmabuf) - 1] = '\0';
-			    (void) add_nma(nmabuf, strlen(nmabuf));
-			}
-		    } else
-			ss = SB_ALL;
-		} else
-		    pn = 0;
-	    }
-	    if (pn) {
-		(void) process_proc_node(lnk ? pbuf : path,
-					 &sb, ss,
-					 (struct stat *)NULL, 0);
-		if (Lf->sf)
-		    link_lfile();
-	    }
-	/*
-	 * Process the PID's execution info.
-	 */
-	    (void) make_proc_path(pidpath, n, &path, &pathl, "exe");
-	    alloc_lfile("txt", -1);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-		(void) memset((void *)&sb, 0, sizeof(sb));
-		lnk = ss = 0;
-		if (!Fwarn) {
-		    if ((errno != ENOENT) || uid) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-			    strerror(errno));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
-		    }
-		    pn = 1;
-		} else
-		    pn = 0;
-	    } else {
-		lnk = 1;
-		if ((scko && !slash) || !scko) {
-		    pn = 1;
-		    if (HasNFS) {
-			if (sv = statsafely(path, &sb)) {
-			    if (lnk) {
-				sv = statEx(pbuf, &sb,  &ss);
-				if (!sv && (ss & SB_DEV) && (ss & SB_INO))
-				    txts = 1;
-			    }
-			}
-		    } else
-			sv = stat(path, &sb);
-		    if (sv) {
-			ss = 0;
-			if (!Fwarn) {
-			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-				strerror(errno));
-			    nmabuf[sizeof(nmabuf) - 1] = '\0';
-			    (void) add_nma(nmabuf, strlen(nmabuf));
-			}
-		    } else {
-			ss = SB_ALL;
-			txts = 1;
-		    }
-		} else
-		    pn = 0;
-	    }
-	    if (pn) {
-		(void) process_proc_node(lnk ? pbuf : path,
-					 &sb, ss,
-					 (struct stat *)NULL, 0);
-		if (Lf->sf)
-		    link_lfile();
-	    }
-	/*
-	 * Process the PID's memory map info.
-	 */
-	    if (!scko) {
-		(void) make_proc_path(pidpath, n, &path, &pathl, "maps");
-		(void) process_proc_map(path, txts ? &sb : (struct stat *)NULL,
-					txts ? ss : 0);
-	    }
-	/*
-	 * Process the PID's file descriptor directory.
-	 */
-	    if ((i = make_proc_path(pidpath, n, &dpath, &dpathl, "fd/")) < 3)
-		continue;
-	    dpath[i - 1] = '\0';
-	    if (!(fdp = opendir(dpath))) {
-		if (!Fwarn) {
-		    (void) snpf(nmabuf, sizeof(nmabuf), "%s (opendir: %s)",
-			dpath, strerror(errno));
-		    alloc_lfile("NOFD", -1);
-		    nmabuf[sizeof(nmabuf) - 1] = '\0';
-		    (void) add_nma(nmabuf, strlen(nmabuf));
-		    link_lfile();
-		}
+	    else if (rv == 0) {
+		(void) process_id(pidpath, n, cmd, uid, pid, ppid, pgid);
 		continue;
 	    }
-	    dpath[i - 1] = '/';
-	    while ((fp = readdir(fdp))) {
-		for (f = fd = n = 0, cp = fp->d_name; *cp; cp++) {
+	/*
+	 * The process is a zombie.  Check for a non-zombie task.
+	 */
+	    (void) make_proc_path(pidpath, n, &taskpath, &taskpathl, "task");
+	    tx = n + 4;
+	    if ((ts = opendir(taskpath))) {
 
-#if	defined(__STDC__)	/* { */
-		    if (!isdigit((unsigned char)*cp))
-#else	/* !defined(__STDC__)	   } { */
-		    if (!isascii(*cp) || !isdigit((unsigned char)*cp))
-#endif	/* defined(__STDC__)	   } */
+	    /*
+	     * Process the PID's tasks (lightweight processes.)  Record the
+	     * open files of the first one whose LWP ID does not match the PID
+	     * and which is not a itself a zombie.
+	     */
+		while ((dp = readdir(ts))) {
 
+		/*
+		 * Get the LWP ID.  Skip the task if its LWP ID matches the
+		 * process PID.
+		 */
+		    if (nm2id(dp->d_name, &lwp, &nl))
+			continue;
+		    if  (lwp == pid)
+			continue;
+		/*
+		 * Check the LWP state.
+		 */
+		    if (read_id_stat(1, path, lwp, &cmd, &ppid, &pgid))
+			continue;
+		/*
+		 * Attempt to record the LWP.
+		 */
+		    if ((tx + 1 + nl + 1) > lwppathl) {
+			lwppathl = tx + 1 + n + 1 + 64;
+			if (lwppath)
+			    lwppath = (char *)realloc((MALLOC_P *)lwppath,
+						      lwppathl);
+			else
+			    lwppath = (char *)malloc((MALLOC_S)lwppathl);
+			if (!lwppath) {
+			    (void) fprintf(stderr,
+				"%s: can't allocate %d task bytes", Pn,
+				lwppathl);
+			    (void) fprintf(stderr,
+				" for \"%s/%s/\"\n", taskpath, dp->d_name);
+			    Exit(1);
+			}
+		    }
+		    (void) snpf(lwppath, lwppathl, "%s/%s/", taskpath,
+			dp->d_name);
+		    if (!process_id(lwppath, (tx + 1 + nl+ 1), cmd, uid, pid,
+				    ppid, pgid))
 		    {
-			f = 1;
 			break;
 		    }
-		    fd = fd * 10 + (*cp - '0');
-		    n++;
-		}
-		if (f)
-		    continue;
-		(void) make_proc_path(dpath, i, &path, &pathl, fp->d_name);
-		(void) alloc_lfile((char *)NULL, fd);
-		if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-		    (void) memset((void *)&sb, 0, sizeof(sb));
-		    lnk = ss = 0;
-		    if (!Fwarn) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-			    strerror(errno));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
-			pn = 1;
-		    } else
-			pn = 0;
-		} else {
-		    lnk = 1;
-		    if ((scko && !slash) || !scko) {
-			pn = 1;
-			if (HasNFS) {
-			    if (lstatsafely(path, &lsb)) {
-				if (lnk)
-				    (void) statEx(pbuf, &lsb, &ls);
-				else
-				    ls = 0;
-				enls = errno;
-			    } else
-				ls = SB_ALL;
-			    if (statsafely(path, &sb)) {
-				if (lnk)
-				    (void) statEx(pbuf, &sb, &ss);
-				else
-				    ss = SB_ALL;
-				enss = errno;
-			    } else
-				ss = SB_ALL;
-			} else {
-			    ls = lstat(path, &lsb) ? 0 : SB_ALL;
-			    enls = errno;
-			    ss = stat(path, &sb) ? 0 : SB_ALL;
-			    enss = errno;
-			}
-			if (!ls && !Fwarn) {
-			    (void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
-				strerror(enls));
-			    nmabuf[sizeof(nmabuf) - 1] = '\0';
-			    (void) add_nma(nmabuf, strlen(nmabuf));
-			}
-			if (!ss && !Fwarn) {
-			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-				strerror(enss));
-			    nmabuf[sizeof(nmabuf) - 1] = '\0';
-			    (void) add_nma(nmabuf, strlen(nmabuf));
-			}
-		    } else
-			pn = 0;
-		}
-		if (pn) {
-		    process_proc_node(lnk ? pbuf : path,
-				      &sb, ss, &lsb, ls);
-		    if (Lf->sf)
-			link_lfile();
 		}
+		(void) closedir(ts);
 	    }
-	    (void) closedir(fdp);
 	}
 }
 
 
+/*
+ * get_fdinfo() - get values from /proc/<PID>fdinfo/FD
+ */
+
+static int
+get_fdinfo(p, fi)
+	char *p;			/* path to fdinfo file */
+	struct l_fdinfo *fi;		/* pointer to local fdinfo values
+					 * return structure */
+{
+	char buf[MAXPATHLEN + 1], *ep, **fp;
+	FILE *fs;
+	int rv = 0;
+	unsigned long ul;
+	unsigned long long ull;
+/*
+ * Signal no values returned (0) if no fdinfo pointer was provided or if the
+ * fdinfo path can't be opened.
+ */
+	if (!fi)
+	    return(0);
+	if (!p || !*p || !(fs = fopen(p, "r")))
+	    return(0);
+/*
+ * Read the fdinfo file.
+ */
+	while (fgets(buf, sizeof(buf), fs)) {
+	    if (get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0) < 2)
+		continue;
+	    if (!fp[0] || !*fp[0] || !fp[1] || !*fp[1])
+		continue;
+	    if (!strcmp(fp[0], "flags:")) {
+
+	    /*
+	     * Process a "flags:" line.
+	     */
+		ep = (char *)NULL;
+		if ((ul = strtoul(fp[1], &ep, 0)) == ULONG_MAX
+		||  !ep || *ep)
+		    continue;
+		fi->flags = (unsigned int)ul;
+		if ((rv |= FDINFO_FLAGS) == FDINFO_ALL)
+		    break;
+	    } else if (!strcmp(fp[0], "pos:")) {
+
+	    /*
+	     * Process a "pos:" line.
+	     */
+		ep = (char *)NULL;
+		if ((ull = strtoull(fp[1], &ep, 0)) == ULLONG_MAX
+		||  !ep || *ep)
+		    continue;
+		fi->pos = (off_t)ull;
+		if ((rv |= FDINFO_POS) == FDINFO_ALL)
+		    break;
+	    }
+	}
+	fclose(fs);
+/*
+ * Signal via the return value what information was obtained. (0 == none)
+ */
+	return(rv);
+}
+
+
 /*
  * getlinksrc() - get the source path name for the /proc/<PID>/fd/<FD> link
  */
 
 
 static int
-getlinksrc(ln, src, srcl, slash)
+getlinksrc(ln, src, srcl)
 	char *ln;			/* link path */
 	char *src;			/* link source path return address */
 	int srcl;			/* length of src[] */
-	int *slash;			/* leading slash flag: 0 = not present
-					 *                     1 = present */
 {
 	char *cp;
 	int ll;
 
-	*slash = 0;
 	if ((ll = readlink(ln, src, srcl - 1)) < 1
 	||  ll >= srcl)
 	    return(-1);
 	src[ll] = '\0';
-	if (*src == '/') {
-	    *slash = 1;
+	if (*src == '/')
 	    return(ll);
-	}
 	if ((cp = strchr(src, ':'))) {
 	    *cp = '\0';
 	    ll = strlen(src);
@@ -511,7 +439,7 @@ void
 initialize()
 {
 	int fd;
-	int off = 0;
+	struct l_fdinfo fi;
 	char path[MAXPATHLEN];
 	struct stat sb;
 /*
@@ -525,7 +453,11 @@ initialize()
 /*
  * Open LSTAT_TEST_FILE and seek to byte LSTAT_TEST_SEEK, then lstat the
  * /proc/<PID>/fd/<FD> for LSTAT_TEST_FILE to see what position is reported.
- * If the result isn't LSTAT_TEST_SEEK, disable offset * reporting.
+ * If the result is LSTAT_TEST_SEEK, enable offset reporting.
+ *
+ * If the result isn't LSTAT_TEST_SEEK, next check the fdinfo file for the
+ * open LSTAT_TEST_FILE file descriptor.  If it exists and contains a "pos:"
+ * value, and if the value is LSTAT_TEST_SEEK, enable offset reporting.
  */
 	if ((fd = open(LSTAT_TEST_FILE, O_RDONLY)) >= 0) {
 	    if (lseek(fd, (off_t)LSTAT_TEST_SEEK, SEEK_SET)
@@ -534,12 +466,20 @@ initialize()
 			    fd);
 		if (!lstat(path, &sb)) {
 		    if (sb.st_size == (off_t)LSTAT_TEST_SEEK)
-			off = 1;
+			OffType = 1;
+		}
+	    }
+	    if (!OffType) {
+		(void) snpf(path, sizeof(path), "%s/%d/fdinfo/%d", PROCFS,
+			    Mypid, fd);
+		if (get_fdinfo(path, &fi) & FDINFO_POS) {
+		    if (fi.pos == (off_t)LSTAT_TEST_SEEK)
+			OffType = 2;
 		}
 	    }
 	    (void) close(fd);
 	}
-	if (!off) {
+	if (!OffType) {
 	    if (Foffset && !Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: can't report offset; disregarding -o.\n",
@@ -547,6 +487,13 @@ initialize()
 	    Foffset = 0;
 	    Fsize = 1;
 	}
+	if (Fsv && (OffType != 2)) {
+	    if (!Fwarn && FsvByf)
+		(void) fprintf(stderr,
+		    "%s: WARNING: can't report file flags; disregarding +f.\n",
+		    Pn);
+	    Fsv = 0;
+	}
 /*
  * Make sure the local mount info table is loaded if doing anything other
  * than just Internet lookups.  (HasNFS is defined during the loading of the
@@ -604,6 +551,472 @@ make_proc_path(pp, pl, np, nl, sf)
 }
 
 
+/*
+ * nm2id() - convert a name to an integer ID
+ */
+
+static int
+nm2id(nm, id, idl)
+	char *nm;			/* pointer to name */
+	int *id;			/* pointer to ID receiver */
+	int *idl;			/* pointer to ID length receiver */
+{
+	register int tid, tidl;
+
+	for (*id = *idl = tid = tidl = 0; *nm; nm++) {
+
+#if	defined(__STDC__)	/* { */
+	    if (!isdigit((unsigned char)*nm))
+#else	/* !defined(__STDC__)	   } { */
+	    if (!isascii(*nm) || !isdigit((unsigned char)*cp))
+#endif	/* defined(__STDC__)	   } */
+
+		{
+		    return(1);
+		}
+		tid = tid * 10 + (int)(*nm - '0');
+		tidl++;
+	}
+	*id = tid;
+	*idl = tidl;
+	return(0);
+}
+
+
+/*
+ * open_proc_stream() -- open a /proc stream
+ */
+
+FILE *
+open_proc_stream(p, m, buf, sz, act)
+	char *p;			/* pointer to path to open */
+	char *m;			/* pointer to mode -- e.g., "r" */
+	char **buf;			/* pointer tp setvbuf() address
+					 * (NULL if none) */
+	size_t *sz;			/* setvbuf() size (0 if none or if
+					 * getpagesize() desired */
+	int act;			/* fopen() failure action:
+					 *     0 : return (FILE *)NULL
+					 *   <>0 : fprintf() an error message
+					 *         and Exit(1)
+					 */
+{
+	FILE *fs;			/* opened stream */
+	static size_t psz = (size_t)0;	/* page size */
+	size_t tsz;			/* temporary size */
+/*
+ * Open the stream.
+ */
+	if (!(fs = fopen(p, m))) {
+	    if (!act)
+		return((FILE *)NULL);
+	    (void) fprintf(stderr, "%s: can't fopen(%s, \"%s\"): %s\n",
+		Pn, p, m, strerror(errno));
+	    Exit(1);
+	}
+/*
+ * Return the stream if no buffer change is required.
+ */
+	if (!buf)
+	    return(fs);
+/*
+ * Determine the buffer size required.
+ */
+	if (!(tsz = *sz)) {
+	    if (!psz)
+		psz = getpagesize();
+	    tsz = psz;
+	}
+/*
+ * Allocate a buffer for the stream, as required.
+ */
+	if (!*buf) {
+	    if (!(*buf = (char *)malloc((MALLOC_S)tsz))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for %s stream buffer\n",
+		    Pn, (int)tsz, p);
+		Exit(1);
+	    }
+	    *sz = tsz;
+	}
+/*
+ * Assign the buffer to the stream.
+ */
+	if (setvbuf(fs, *buf, _IOFBF, tsz)) {
+	    (void) fprintf(stderr, "%s: setvbuf(%s)=%d failure: %s\n",
+		Pn, p, (int)tsz, strerror(errno));
+	    Exit(1);
+	}
+	return(fs);
+}
+
+
+/*
+ * process_id - process ID: PID or LWP
+ *
+ * return:  0 == ID processed
+ *          1 == ID not processed
+ */
+
+static int
+process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
+	char *idp;			/* pointer to ID's path */
+	int idpl;			/* pointer to ID's path length */
+	char *cmd;			/* pointer to ID's command */
+	UID_ARG uid;			/* ID's UID */
+	int pid;			/* ID's PID */
+	int ppid;			/* parent PID */
+	int pgid;			/* parent GID */
+{
+	int av;
+	static char *dpath = (char *)NULL;
+	static int dpathl = 0;
+	short enls, enss, lnk, oty, pn, pss, sf;
+	int fd, i, ls, n, ss, sv;
+	struct l_fdinfo fi;
+	DIR *fdp;
+	struct dirent *fp;
+	static char *ipath = (char *)NULL;
+	static int ipathl = 0;
+	int j = 0;
+	struct stat lsb, sb;
+	char nmabuf[MAXPATHLEN + 1], pbuf[MAXPATHLEN + 1];
+	static char *path = (char *)NULL;
+	static int pathl = 0;
+	static char *pathi = (char *)NULL;
+	static int pathil = 0;
+	int txts = 0;
+
+#if	defined(HASSELINUX)
+	cntxlist_t *cntxp;
+#endif	/* defined(HASSELINUX) */
+
+/*
+ * See if process is excluded.
+ */
+	if (is_proc_excl(pid, pgid, uid, &pss, &sf)
+	||  is_cmd_excl(cmd, &pss, &sf))
+	    return(1);
+/*
+ * Set the socket processing options.
+ */
+	if (Cscko) {
+	    if (sf & SELPROC)
+		Scko = 0;
+	    else
+		Scko = 1;
+	}
+	alloc_lproc(pid, pgid, ppid, uid, cmd, (int)pss, (int)sf);
+	Plf = (struct lfile *)NULL;
+/*
+ * Process the ID's current working directory info.
+ */
+	(void) make_proc_path(idp, idpl, &path, &pathl, "cwd");
+	alloc_lfile(CWD, -1);
+	if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    if (!Fwarn) {
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		lnk = ss = 0;
+		(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+		    strerror(errno));
+		nmabuf[sizeof(nmabuf) - 1] = '\0';
+		(void) add_nma(nmabuf, strlen(nmabuf));
+		pn = 1;
+	    } else
+		pn = 0;
+	} else {
+	    lnk = 1;
+	    ss = SB_ALL;
+	    if (HasNFS) {
+		if ((sv = statsafely(path, &sb)))
+		    sv = statEx(pbuf, &sb, &ss);
+	    } else
+		sv = stat(path, &sb);
+	    if (sv) {
+		ss = 0;
+		if (!Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+	    }
+	    if (Scko) {
+		if ((ss & SB_MODE) && ((sb.st_mode & S_IFMT) == S_IFSOCK))
+		    pn = 1;
+		else
+		    pn = 0;
+	    } else
+		pn = 1;
+	}
+	if (pn) {
+	    (void) process_proc_node(lnk ? pbuf : path,
+				     &sb, ss,
+				     (struct stat *)NULL, 0);
+	    if (Lf->sf)
+		link_lfile();
+	}
+/*
+ * Process the ID's root directory info.
+ */
+	(void) make_proc_path(idp, idpl, &path, &pathl, "root");
+	alloc_lfile(RTD, -1);
+	if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    if (!Fwarn) {
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		lnk = ss = 0;
+		(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+		    strerror(errno));
+		nmabuf[sizeof(nmabuf) - 1] = '\0';
+		(void) add_nma(nmabuf, strlen(nmabuf));
+		pn = 1;
+	    } else
+		pn = 0;
+	} else {
+	    lnk = 1;
+	    ss = SB_ALL;
+	    if (HasNFS) {
+		if ((sv = statsafely(path, &sb)))
+		    sv = statEx(pbuf, &sb, &ss);
+	    } else
+		sv = stat(path, &sb);
+	    if (sv) {
+		ss = 0;
+		if (!Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+	    }
+	    if (Scko) {
+		if ((ss & SB_MODE) && ((sb.st_mode & S_IFMT) == S_IFSOCK))
+		    pn = 1;
+		else
+		    pn = 0;
+	    } else
+		pn = 1;
+	}
+	if (pn) {
+	    (void) process_proc_node(lnk ? pbuf : path,
+				     &sb, ss,
+				     (struct stat *)NULL, 0);
+	    if (Lf->sf)
+		link_lfile();
+	}
+/*
+ * Process the ID's execution info.
+ */
+	(void) make_proc_path(idp, idpl, &path, &pathl, "exe");
+	alloc_lfile("txt", -1);
+	if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    (void) memset((void *)&sb, 0, sizeof(sb));
+	    lnk = ss = 0;
+	    if (!Fwarn) {
+		if ((errno != ENOENT) || uid) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+		pn = 1;
+	    } else
+		pn = 0;
+	 } else {
+	    lnk = 1;
+	    ss = SB_ALL;
+	    if (HasNFS) {
+		if ((sv = statsafely(path, &sb))) {
+		    sv = statEx(pbuf, &sb,  &ss);
+		    if (!sv && (ss & SB_DEV) && (ss & SB_INO))
+			txts = 1;
+		}
+	    } else
+		sv = stat(path, &sb);
+	    if (sv) {
+		ss = 0;
+		if (!Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+	    } else
+		txts = 1;
+	    if (Scko) {
+		if ((ss & SB_MODE) && ((sb.st_mode & S_IFMT) == S_IFSOCK))
+		    pn = 1;
+		else
+		    pn = 0;
+	    } else
+		pn = 1;
+	}
+	if (pn) {
+		(void) process_proc_node(lnk ? pbuf : path,
+					 &sb, ss,
+					 (struct stat *)NULL, 0);
+		if (Lf->sf)
+		    link_lfile();
+	}
+/*
+ * Process the ID's memory map info.
+ */
+	if (!Scko) {
+	    (void) make_proc_path(idp, idpl, &path, &pathl, "maps");
+	    (void) process_proc_map(path, txts ? &sb : (struct stat *)NULL,
+				    txts ? ss : 0);
+	}
+
+#if	defined(HASSELINUX)
+/*
+ * Process the PID's SELinux context.
+ */
+	if (Fcntx) {
+
+	/*
+	 * If the -Z (cntx) option was specified, match the valid contexts.
+	 */
+	    errno = 0;
+	    if (getpidcon(pid, &Lp->cntx) == -1) {
+		Lp->cntx = (char *)NULL;
+		if (!Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf),
+			"(getpidcon: %s)", strerror(errno));
+		    if (!(Lp->cntx = strdup(nmabuf))) {
+			(void) fprintf(stderr,
+			    "%s: no context error space: PID %ld",
+			    Pn, (long)Lp->pid);
+			Exit(1);
+		    }
+		}
+	    } else if (CntxArg) {
+
+	    /*
+	     * See if context includes the process.
+	     */
+		for (cntxp = CntxArg; cntxp; cntxp = cntxp->next) {
+		    if (cmp_cntx_eq(Lp->cntx, cntxp->cntx)) {
+			cntxp->f = 1;
+			Lp->pss |= PS_PRI;
+			Lp->sf |= SELCNTX;
+			break;
+		    }
+		}
+	    }
+	}
+#endif	/* defined(HASSELINUX) */
+
+/*
+ * Process the ID's file descriptor directory.
+ */
+	if ((i = make_proc_path(idp, idpl, &dpath, &dpathl, "fd/")) < 3)
+	    return(0);
+	dpath[i - 1] = '\0';
+	if ((OffType == 2)
+	&&  ((j = make_proc_path(idp, idpl, &ipath, &ipathl, "fdinfo/")) >= 7))
+	    oty = 1;
+	else
+	    oty = 0;
+	if (!(fdp = opendir(dpath))) {
+	    if (!Fwarn) {
+		(void) snpf(nmabuf, sizeof(nmabuf), "%s (opendir: %s)",
+		    dpath, strerror(errno));
+		alloc_lfile("NOFD", -1);
+		nmabuf[sizeof(nmabuf) - 1] = '\0';
+		(void) add_nma(nmabuf, strlen(nmabuf));
+		link_lfile();
+	    }
+	    return(0);
+	}
+	dpath[i - 1] = '/';
+	while ((fp = readdir(fdp))) {
+	    if (nm2id(fp->d_name, &fd, &n))
+		continue;
+	    (void) make_proc_path(dpath, i, &path, &pathl, fp->d_name);
+	    (void) alloc_lfile((char *)NULL, fd);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		lnk = ss = 0;
+		if (!Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = 1;
+		if (HasNFS) {
+		    if (lstatsafely(path, &lsb)) {
+			(void) statEx(pbuf, &lsb, &ls);
+		        enls = errno;
+		    } else {
+			enls = 0;
+			ls = SB_ALL;
+		    }
+		    if (statsafely(path, &sb)) {
+			(void) statEx(pbuf, &sb, &ss);
+			enss = errno;
+		    } else {
+			enss = 0;
+			ss = SB_ALL;
+		    }
+		} else {
+		    ls = lstat(path, &lsb) ? 0 : SB_ALL;
+		    enls = errno;
+		    ss = stat(path, &sb) ? 0 : SB_ALL;
+		    enss = errno;
+		}
+		if (!ls && !Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
+			strerror(enls));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+		if (!ss && !Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			strerror(enss));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+		if (Scko) {
+		    if ((ss & SB_MODE) && ((sb.st_mode & S_IFMT) == S_IFSOCK))
+			pn = 1;
+		    else
+			pn = 0;
+		} else
+		    pn = 1;
+	    }
+	    if (pn) {
+		if (oty) {
+		    (void) make_proc_path(ipath, j, &pathi, &pathil,
+					  fp->d_name);
+		    if ((av = get_fdinfo(pathi, &fi)) & FDINFO_POS) {
+			ls |= SB_SIZE;
+			lsb.st_size = fi.pos;
+		    } else
+			ls &= ~SB_SIZE;
+
+#if	!defined(HASNOFSFLAGS)
+		    if ((av & FDINFO_FLAGS) && (Fsv & FSV_FG)) {
+			Lf->ffg = (long)fi.flags;
+			Lf->fsv |= FSV_FG;
+		     }
+# endif	/* !defined(HASNOFSFLAGS) */
+
+		}
+		process_proc_node(lnk ? pbuf : path,
+				  &sb, ss, &lsb, ls);
+		if (Lf->sf)
+		    link_lfile();
+	    }
+	}
+	(void) closedir(fdp);
+	return(0);
+}
+
+
 /*
  * process_proc_map() - process the memory map of a process
  */
@@ -614,10 +1027,11 @@ process_proc_map(p, s, ss)
 	struct stat *s;			/* executing text file state buffer */
 	int ss;				/* *s status -- i.e., SB_* values */
 {
-	char buf[MAXPATHLEN + 1], *ep, **fp, nmabuf[MAXPATHLEN + 1];
+	char buf[MAXPATHLEN + 1], *ep, fmtbuf[32], **fp, nmabuf[MAXPATHLEN + 1];
 	dev_t dev;
-	int en, i, mss, nf, sv;
-	ino_t inode;
+	int ds, en, i, mss, nf, sv;
+	int eb = 6;
+	INODETYPE inode;
 	MALLOC_S len;
 	long maj, min;
 	FILE *ms;
@@ -625,20 +1039,33 @@ process_proc_map(p, s, ss)
 	struct stat sb;
 	struct saved_map {
 	    dev_t dev;
-	    ino_t inode;
+	    INODETYPE inode;
 	};
 	static struct saved_map *sm = (struct saved_map *)NULL;
 	static int sma = 0;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 /*
- * Open and read the /proc/<pid>/maps file.
+ * Open the /proc/<pid>/maps file, assign a page size buffer to its stream,
+ * and read it/
  */
-	if (!(ms = fopen(p, "r")))
+	if (!(ms = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
 	    return;
 	while (fgets(buf, sizeof(buf), ms)) {
-	    if ((nf = get_fields(buf, ":", &fp)) < 7)
+	    if ((nf = get_fields(buf, ":", &fp, &eb, 1)) < 7)
 		continue;			/* not enough fields */
 	    if (!fp[6] || !*fp[6])
 		continue;			/* no path name */
+	/*
+	 * See if the path ends in " (deleted)".  If it does, strip the
+	 * " (deleted)" characters and remember that they were there.
+	 */
+	    if (((ds = (int)strlen(fp[6])) > 10)
+	    &&  !strcmp(fp[6] + ds - 10, " (deleted)"))
+	    {
+		*(fp[6] + ds - 10) = '\0';
+	    } else
+		ds = 0;
 	/*
 	 * Assemble the major and minor device numbers.
 	 */
@@ -653,18 +1080,24 @@ process_proc_map(p, s, ss)
 	    ||  !ep || *ep)
 		continue;
 	/*
-	 * Assemble the inode number.
+	 * Assemble the device and inode numbers.  If they are both zero, skip
+	 * the entry.
 	 */
+	    dev = (dev_t)makedev((int)maj, (int)min);
 	    if (!fp[5] || !*fp[5])
 		continue;
-	    inode = (ino_t)atoi(fp[5]);
+	    ep = (char *)NULL;
+	    if ((inode = strtoull(fp[5], &ep, 0)) == ULLONG_MAX
+	    ||  !ep || *ep)
+		continue;
+	    if (!dev && !inode)
+		continue;
 	/*
 	 * See if the device + inode pair match that of the executable.
 	 * If they do, skip this map entry.
 	 */
-	    dev = (dev_t)makedev((int)maj, (int)min);
 	    if (s && (ss & SB_DEV) && (ss & SB_INO)
-	    &&  (dev == s->st_dev) && (inode == s->st_ino))
+	    &&  (dev == s->st_dev) && (inode == (INODETYPE)s->st_ino))
 		continue;
 	/*
 	 * See if this device + inode pair has already been processed as
@@ -715,10 +1148,10 @@ process_proc_map(p, s, ss)
 	     */
 		(void) memset((void *)&sb, 0, sizeof(sb));
 		sb.st_dev = dev;
-		sb.st_ino = inode;
+		sb.st_ino = (ino_t)inode;
 		sb.st_mode = S_IFREG;
 		mss = SB_DEV | SB_INO | SB_MODE;
-		if ((nf >= 8) && !strcmp(fp[7], "(deleted)"))
+		if (ds)
 		    alloc_lfile("DEL", -1);
 		else {
 		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
@@ -726,7 +1159,7 @@ process_proc_map(p, s, ss)
 		    nmabuf[sizeof(nmabuf) - 1] = '\0';
 		    (void) add_nma(nmabuf, strlen(nmabuf));
 		}
-	    } else if ((sb.st_dev != dev) || (sb.st_ino != inode)) {
+	    } else if ((sb.st_dev != dev) || ((INODETYPE)sb.st_ino != inode)) {
 
 	    /*
 	     * The stat(2) device and inode numbers don't match those obtained
@@ -738,7 +1171,7 @@ process_proc_map(p, s, ss)
 	     * Manufacture a partial stat(2) reply from the maps file
 	     * information.
 	     */
-		if ((nf >= 8) && !strcmp(fp[7], "(deleted)"))
+		if (ds)
 		    alloc_lfile("DEL", -1);
 		else if (!Fwarn) {
 		    char *sep;
@@ -747,22 +1180,24 @@ process_proc_map(p, s, ss)
 			(void) snpf(nmabuf, sizeof(nmabuf),
 			    "(path dev=%d,%d%s",
 			    GET_MAJ_DEV(sb.st_dev), GET_MIN_DEV(sb.st_dev),
-			    (sb.st_ino == inode) ? ")" : ",");
+			    ((INODETYPE)sb.st_ino == inode) ? ")" : ",");
 			nmabuf[sizeof(nmabuf) - 1] = '\0';
 			(void) add_nma(nmabuf, strlen(nmabuf));
 			sep = "";
 		    } else
 			sep = "(path ";
-		    if (sb.st_ino != inode) {
-			(void) snpf(nmabuf, sizeof(nmabuf),
-			    "%sinode=%ld)", sep, sb.st_ino);
+		    if ((INODETYPE)sb.st_ino != inode) {
+			(void) snpf(fmtbuf, sizeof(fmtbuf), "%%sinode=%s)",
+			    InodeFmt_d);
+			(void) snpf(nmabuf, sizeof(nmabuf), fmtbuf,
+			    sep, (INODETYPE)sb.st_ino);
 			nmabuf[sizeof(nmabuf) - 1] = '\0';
 			(void) add_nma(nmabuf, strlen(nmabuf));
 		    }
 		}
 		(void) memset((void *)&sb, 0, sizeof(sb));
 		sb.st_dev = dev;
-		sb.st_ino = inode;
+		sb.st_ino = (ino_t)inode;
 		sb.st_mode = S_IFREG;
 		mss = SB_DEV | SB_INO | SB_MODE;
 	    } else
@@ -779,42 +1214,55 @@ process_proc_map(p, s, ss)
 
 
 /*
- * read_proc_stat() - read process status
+ * read_id_stat() - read ID (PID or LWP ID) status
+ *
+ * return: -1 == ID is a zombie
+ *          0 == ID OK
+ *          1 == ID unavailable
  */
 
 static int
-read_proc_stat(p, pid, cmd, ppid, pgid)
+read_id_stat(ty, p, id, cmd, ppid, pgid)
+	int ty;				/* type: 0 == PID, 1 == LWP */
 	char *p;			/* path to status file */
-	int pid;			/* PID */
+	int id;				/* ID: PID or LWP */
 	char **cmd;			/* malloc'd command name */
-	int *ppid;			/* parent PID */
-	int *pgid;			/* process group ID */
+	int *ppid;			/* returned parent PID for PID type */
+	int *pgid;			/* returned process group ID for PID
+					 * type */
 {
 	char buf[MAXPATHLEN], *cp, *cp1, **fp;
 	static char *cbf = (char *)NULL;
 	static MALLOC_S cbfa = 0;
 	FILE *fs;
 	MALLOC_S len;
-	int nf, tpid;
+	int nf;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 /*
- * Open the stat file path and read its first line.
+ * Open the stat file path, assign a page size buffer to its stream,
+ * and read the file's first line.
  */
-	if (!(fs = fopen(p, "r")))
+	if (!(fs = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
 	    return(1);
 	cp = fgets(buf, sizeof(buf), fs);
 	(void) fclose(fs);
 	if (!cp)
 	    return(1);
 /*
- * Separate the line into fields on white space separators.
+ * Separate the line into fields on white space separators.  Expect five fields
+ * for a PID type and three for an LWP type.
  */
-	if ((nf = get_fields(buf, (char *)NULL, &fp)) < 5)
+	if ((nf = get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0))
+	<  (ty ? 5 : 3))
+	{
 	    return(1);
+	}
 /*
  * Convert the first field to an integer; its conversion must match the
- * PID argument.
+ * ID argument.
  */
-	if (atoi(fp[0]) != pid)
+	if (!fp[0] || (atoi(fp[0]) != id))
 	    return(1);
 /*
  * Get the command name from the second field.  Strip a starting '(' and
@@ -843,24 +1291,23 @@ read_proc_stat(p, pid, cmd, ppid, pgid)
 	(void) snpf(cbf, len, "%s", cp);
 	*cmd = cbf;
 /*
- * Check the state in the third field.  If it is 'Z', this is a defunct
- * process and should be skipped.
+ * If the type is PID, convert and return parent process (fourth field)
+ * and process group (fifth field) IDs.
  */
-	if (!fp[2] || !strcmp(fp[2], "Z"))
-	    return(1);
+	if (!ty) {
+	    if (fp[3] && *fp[3])
+		*ppid = atoi(fp[3]);
+	    else
+		return(1);
+	    if (fp[4] && *fp[4])
+		*pgid = atoi(fp[4]);
+	    else
+		return(1);
+	}
 /*
- * Convert and return parent process (fourth field) and process group (fifth
- * field) IDs.
+ * Check the state in the third field.  If it is 'Z', return that indication.
  */
-	if (fp[3] && *fp[3])
-	    *ppid = atoi(fp[3]);
-	else
-	    return(1);
-	if (fp[4] && *fp[4])
-	    *pgid = atoi(fp[4]);
-	else
-	    return(1);
-	return(0);
+	return((fp[2] && !strcmp(fp[2], "Z")) ? -1 : 0);
 }
 
 
