@@ -32,12 +32,16 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.21 2001/11/01 20:21:18 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.25 2008/04/15 13:24:19 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+#if	defined(HASWIDECHAR) && defined(WIDECHARINCL)
+#include WIDECHARINCL
+#endif	/* defined(HASWIDECHAR) && defined(WIDECHARINCL) */
+
 
 /*
  * Local definitions
@@ -58,18 +62,6 @@ _PROTOTYPE(static int dostat,(char *path, char *buf, int len));
 _PROTOTYPE(static int doreadlink,(char *path, char *buf, int len));
 _PROTOTYPE(static int doinchild,(int (*fn)(), char *fp, char *rbuf, int rbln));
 
-# if	!defined(GET_MAX_FD)
-/*
- * This is not strictly a prototype, but GET_MAX_FD is the name of the
- * function that, in lieu of getdtablesize(), returns the maximum file
- * descriptor plus one (or file descriptor count).  GET_MAX_FD may be
- * defined in the dialect's machine.h.  If it is not, the following
- * selects getdtablesize().
- */
-
-#define	GET_MAX_FD	getdtablesize
-# endif	/* !defined(GET_MAX_FD) */
-
 #if	defined(HASINTSIGNAL)
 _PROTOTYPE(static int handleint,(int sig));
 #else	/* !defined(HASINTSIGNAL) */
@@ -141,7 +133,7 @@ build_Nl(d)
 void
 childx()
 {
-	int at, sx;
+	static int at, sx;
 	pid_t wpid;
 
 	if (Cpid > 1) {
@@ -232,9 +224,9 @@ compdev(a1, a2)
 	    return(-1);
 	if ((dev_t)((*p1)->rdev) > (dev_t)((*p2)->rdev))
 	    return(1);
-	if ((ino_t)((*p1)->inode) < (ino_t)((*p2)->inode))
+	if ((INODETYPE)((*p1)->inode) < (INODETYPE)((*p2)->inode))
 	    return(-1);
-	if ((ino_t)((*p1)->inode) > (ino_t)((*p2)->inode))
+	if ((INODETYPE)((*p1)->inode) > (INODETYPE)((*p2)->inode))
 	    return(1);
 	return(strcmp((*p1)->name, (*p2)->name));
 }
@@ -298,7 +290,7 @@ doinchild(fn, fp, rbuf, rbln)
 		 */
 
 		    int fd, nd, r_al, r_rbln;
-		    char r_arg[MAXPATHLEN], r_rbuf[MAXPATHLEN];
+		    char r_arg[MAXPATHLEN+1], r_rbuf[MAXPATHLEN+1];
 		    int (*r_fn)();
 		/*
 		 * Close all open file descriptors except Pipes[0] and
@@ -345,7 +337,7 @@ doinchild(fn, fp, rbuf, rbln)
 			||  write(Pipes[3], r_rbuf, r_rbln) != r_rbln)
 			    break;
 		    }
-		    _exit(0);
+		    (void) _exit(0);
 		}
 	    /*
 	     * Continue in the parent process to finish the setup.
@@ -808,18 +800,18 @@ char *
 Readlink(arg)
 	char *arg;			/* argument to be interpreted */
 {
-	char abuf[MAXPATHLEN];
+	char abuf[MAXPATHLEN+1];
 	int alen;
 	char *ap;
 	char *argp1, *argp2;
 	int i, len, llen, slen;
-	char lbuf[MAXPATHLEN];
+	char lbuf[MAXPATHLEN+1];
 	static char *op = (char *)NULL;
 	static int ss = 0;
 	char *s1;
 	static char **stk = (char **)NULL;
 	static int sx = 0;
-	char tbuf[MAXPATHLEN];
+	char tbuf[MAXPATHLEN+1];
 /*
  * See if avoiding kernel blocks.
  */
@@ -843,7 +835,7 @@ Readlink(arg)
 	for (alen = 0, ap = abuf, argp1 = argp2 = arg; *argp2; argp1 = argp2 ) {
 	    for (argp2 = argp1 + 1; *argp2 && *argp2 != '/'; argp2++)
 		;
-	    if ((len = argp2 - arg) >= MAXPATHLEN) {
+	    if ((len = argp2 - arg) >= (int)sizeof(tbuf)) {
 
 path_too_long:
 		if (!Fwarn) {
@@ -1199,12 +1191,44 @@ safestrprt(sp, fs, flags)
 					 */
 {
 	char c;
+	int lnc, lnt, sl;
+
+#if	defined(HASWIDECHAR)
+	wchar_t w;
+	int wcmx = MB_CUR_MAX;
+#else	/* !defined(HASWIDECHAR) */
+	static int wcmx = 1;
+#endif	/* defined(HASWIDECHAR) */
 
 	c = (flags & 2) ? ' ' : '\0';
 	if (flags & 4)
 	    putc('"', fs);
 	if (sp) {
-	    for (; *sp; sp++) {
+	    for (sl = strlen(sp); *sp; sl -= lnc, sp += lnc) {
+
+#if	defined(HASWIDECHAR)
+		if (wcmx > 1) {
+		    lnc = mblen(sp, sl);
+		    if (lnc > 1) {
+			if ((mbtowc(&w, sp, sl) == lnc) && iswprint(w)) {
+			    for (lnt = 0; lnt < lnc; lnt++) {
+				putc((int)*(sp + lnt), fs);
+			    }
+			} else {
+			    for (lnt = 0; lnt < lnc; lnt++) {
+			        fputs(safepup((unsigned int)*(sp + lnt),
+					      (int *)NULL), fs);
+			    }
+			}
+			continue;
+		    } else
+			lnc = 1;
+		} else
+		    lnc = 1;
+#else	/* !defined(HASWIDECHAR) */
+		lnc = 1;
+#endif	/* defined(HASWIDECHAR) */
+
 		if (isprint((unsigned char)*sp) && *sp != c)
 		    putc((int)(*sp & 0xff), fs);
 		else {
@@ -1346,7 +1370,7 @@ x2dev(s, d)
 	char *s;			/* ASCII string */
 	dev_t *d;			/* device receptacle */
 {
-	char c, *cp, *cp1;
+	char *cp, *cp1;
 	int n;
 	dev_t r;
 
